/****************************************************************************
 *
 * Rohde & Schwarz DVSG Digital Video Signal Generator instrument driver 
 * include file
 ****************************************************************************/

#ifndef __RSDVSG_HEADER
#define __RSDVSG_HEADER

#include "rsidr_core.h"
#include "rsdvsg_attributes.h"
#include "rsdvsg_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/

#define RSDVSG_SIMULATION_ID_QUERY      "Rohde&Schwarz,DVSG,123456/789,1.0.10"  /* Default definition of ID Query Response for simulation */
#define RSDVSG_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSDVSG_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSDVSG_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSDVSG_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
//#define IEEE_488_2_ERROR_MASK             0x3C    /* 488.2 Event Status Register (ESR) Bits */
#define IEEE_488_2_ERROR_MASK             0x4    /* 488.2 STB Bits */   
#define IEEE_488_2_ESE_MASK               0x3D    /* 488.2 Event Status Register (ESE) Bits */
#define MAX_OPC_TIME                    60000L

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSDVSG_CLASS_DRIVER_DESCRIPTION           ""
#define RSDVSG_CLASS_DRIVER_PREFIX                ""
#define RSDVSG_CLASS_DRIVER_VENDOR                ""
#define RSDVSG_CLASS_DRIVER_REVISION              ""
#define RSDVSG_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSDVSG_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSDVSG_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSDVSG_SPECIFIC_DRIVER_PREFIX             "RSDVSG"
#define RSDVSG_SPECIFIC_DRIVER_LOCATOR            ""
#define RSDVSG_SPECIFIC_DRIVER_VENDOR             ""
#define RSDVSG_MAJOR_VERSION                      2L    /* Instrument driver major version          */
#define RSDVSG_MINOR_VERSION                      0L   /* Instrument driver minor version          */
#define RSDVSG_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSDVSG_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSDVSG_GROUP_CAPABILITIES                 ""
#define RSDVSG_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *------------------------------ Useful Macros -----------------------------*
 ****************************************************************************/


/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSDVSG_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSDVSG_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSDVSG_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSDVSG_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSDVSG_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSDVSG_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSDVSG_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSDVSG_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSDVSG_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSDVSG_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSDVSG_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSDVSG_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSDVSG_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSDVSG_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSDVSG_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSDVSG_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSDVSG_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSDVSG_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSDVSG_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSDVSG_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSDVSG_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSDVSG_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSDVSG_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSDVSG_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSDVSG_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSDVSG_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSDVSG_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSDVSG_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSDVSG_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSDVSG_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSDVSG_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME                                    /* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSDVSG_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 169
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_STATUS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 170
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_STATUS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 171
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_SOURCE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 172
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_FILE_INFO                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 173
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_ORIGINAL_LOOP_TIME               RS_SPECIFIC_PUBLIC_ATTR_BASE + 174
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_ORIGINAL_TS_DATA_RATE            RS_SPECIFIC_PUBLIC_ATTR_BASE + 175
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_FILE_START_POSITION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 176
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_FILE_STOP_POSITION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 177
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_CURRENT_PLAY_TIME                RS_SPECIFIC_PUBLIC_ATTR_BASE + 178
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_TV_STANDARD                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 179
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_SERVICE_LIST_INDEX                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 180
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_SERVICE_LIST_MAXIMUM_NUMBER      RS_SPECIFIC_PUBLIC_ATTR_BASE + 181
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_SERVICE_LIST_INFO                RS_SPECIFIC_PUBLIC_ATTR_BASE + 182
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_VIDEO_LIST_INDEX                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 183
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_LIST_MAXIMUM_NUMBER        RS_SPECIFIC_PUBLIC_ATTR_BASE + 184
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_LIST_INFO                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 185
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_CODING                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 186
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_ES_DATA_RATE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 187
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_FRAME_RATE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 188
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_PICTURE_SIZE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 189
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_PICTURE_FORMAT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 190
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_AUDIO_LIST_INDEX                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 191
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_AUDIO_LIST_MAXIMUM_NUMBER        RS_SPECIFIC_PUBLIC_ATTR_BASE + 192
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_AUDIO_LIST_INFO                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 193
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_AUDIO_CODING                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 194
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_AUDIO_ES_DATA_RATE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 195
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_AUDIO_SAMPLING_RATE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 196
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_VIDEO_RESOLUTION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 197
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_VIDEO_ASPECT_RATIO                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 198
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_CONFORMITY             RS_SPECIFIC_PUBLIC_ATTR_BASE + 199
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_HDMI_HDCP                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 200
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_HDMI_DVI                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 201
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_HD_SDI_RESOLUTION      RS_SPECIFIC_PUBLIC_ATTR_BASE + 202
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_HD_SDI_EMBEDDED_AUDIO  RS_SPECIFIC_PUBLIC_ATTR_BASE + 203
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_COMPOSITE_NTSC_PEDESTAL      RS_SPECIFIC_PUBLIC_ATTR_BASE + 204
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_COMPOSITE_MACRO_VISION       RS_SPECIFIC_PUBLIC_ATTR_BASE + 205
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_COLOR_SYSTEM           RS_SPECIFIC_PUBLIC_ATTR_BASE + 206
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_COMPONENT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 207
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_COMPONENT_HDTV_SYNC    RS_SPECIFIC_PUBLIC_ATTR_BASE + 208
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_MONITOR_INPUT_SELECT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 209
#define RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_SCART_FORMAT_WSS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 210
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_YC                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 211
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_VGA                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 212
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_TS_ASI                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 213
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_TS_PACKET_LENGTH       RS_SPECIFIC_PUBLIC_ATTR_BASE + 214
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_RGB_SYNC               RS_SPECIFIC_PUBLIC_ATTR_BASE + 215
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 216
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_STATUS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 217
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_STATUS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 218
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_SOURCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 219
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_FRAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 220
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_FRAME_RATE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 221
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_SAMPLING_RATE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 222
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_PIXELSHIFT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 223
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VERTICAL_PIXELSHIFT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 224
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_ADJUSTMENT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 225
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_RESOLUTION_CLIPPING           RS_SPECIFIC_PUBLIC_ATTR_BASE + 226
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_RGB_VIDEO_OVERALL_AMPLIFICATION     RS_SPECIFIC_PUBLIC_ATTR_BASE + 227
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_RGB_VIDEO_R_LEVEL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 228
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_RGB_VIDEO_G_LEVEL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 229
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_RGB_VIDEO_B_LEVEL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 230
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_OFFSET                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 231
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_YCRCB_VIDEO_CHROMA_AMPLIFICATION    RS_SPECIFIC_PUBLIC_ATTR_BASE + 232
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_YCRCB_VIDEO_OVERALL_AMPLIFICATION   RS_SPECIFIC_PUBLIC_ATTR_BASE + 233
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_YCRCB_VIDEO_Y_LEVEL                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 234
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_YCRCB_VIDEO_CB_PB_LEVEL             RS_SPECIFIC_PUBLIC_ATTR_BASE + 235
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_YCRCB_VIDEO_CR_PR_LEVEL             RS_SPECIFIC_PUBLIC_ATTR_BASE + 236
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_SOURCE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 237
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_CHANNEL_ALLOCATION            RS_SPECIFIC_PUBLIC_ATTR_BASE + 238
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_LEVEL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 239
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_RESOLUTION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 240
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_FREQUENCY                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 241
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_CONFORMITY          RS_SPECIFIC_PUBLIC_ATTR_BASE + 242
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_HDMI_HDCP                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 243
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_HDMI_DVI                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 244
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_COMPOSITE_NTSC_PEDESTAL   RS_SPECIFIC_PUBLIC_ATTR_BASE + 245
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_COMPOSITE_MACRO_VISION    RS_SPECIFIC_PUBLIC_ATTR_BASE + 246
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_COMPOSITE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 247
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_COMPONENT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 248
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_COMPONENT_HDTV_SYNC RS_SPECIFIC_PUBLIC_ATTR_BASE + 249
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_YC                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 250
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_SCART_MONITOR_INPUT_SELECT RS_SPECIFIC_PUBLIC_ATTR_BASE + 251
#define RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_SCART_FORMAT_WSS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 252
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_VGA                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 253
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_TS_PACKET_LENGTH    RS_SPECIFIC_PUBLIC_ATTR_BASE + 254
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_ASI                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 255
#define RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_RGB_SYNC            RS_SPECIFIC_PUBLIC_ATTR_BASE + 256
#define RSDVSG_ATTR_TS_PLAYER_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 257
#define RSDVSG_ATTR_TS_APPLICATION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 258
#define RSDVSG_ATTR_TS_PLAYER_AND_RECORDER_STATUS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 259
#define RSDVSG_ATTR_QUERY_TS_PLAYER_AND_RECORDER_STATUS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 260
#define RSDVSG_ATTR_TS_PLAYER_SOURCE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 261
#define RSDVSG_ATTR_QUERY_TS_PLAYER_FILE_DATE_SIZE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 262
#define RSDVSG_ATTR_QUERY_TS_PLAYER_ORIGINAL_LOOP_TIME                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 263
#define RSDVSG_ATTR_QUERY_TS_PLAYER_ORIGINAL_TS_DATA_RATE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 264
#define RSDVSG_ATTR_QUERY_TS_PLAYER_CURRENT_PLAY_TIME                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 265
#define RSDVSG_ATTR_TS_PLAYER_TIMING_SETTINGS_TS_DATA_RATE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 266
#define RSDVSG_ATTR_TS_PLAYER_TIMING_SETTINGS_PLAY_WINDOW_START             RS_SPECIFIC_PUBLIC_ATTR_BASE + 267
#define RSDVSG_ATTR_TS_PLAYER_TIMING_SETTINGS_PLAY_WINDOW_STOP              RS_SPECIFIC_PUBLIC_ATTR_BASE + 268
#define RSDVSG_ATTR_TS_PLAYER_BYTE_RATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 269
#define RSDVSG_ATTR_TS_PLAYER_NULLPACKET_STUFFING                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 270
#define RSDVSG_ATTR_TS_PLAYER_PCR_JITTER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 271
#define RSDVSG_ATTR_TS_PLAYER_PCR_JITTER_WAVEFORM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 272
#define RSDVSG_ATTR_TS_PLAYER_PCR_JITTER_AMPLITUDE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 273
#define RSDVSG_ATTR_TS_PLAYER_PCR_JITTER_FREQUENCY                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 274
#define RSDVSG_ATTR_TS_PLAYER_SEAMLESS_LOOP_CONTINUITY_COUNTER              RS_SPECIFIC_PUBLIC_ATTR_BASE + 166
#define RSDVSG_ATTR_TS_PLAYER_SEAMLESS_LOOP_PCR_DTS_PTS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 167
#define RSDVSG_ATTR_TS_PLAYER_SEAMLESS_LOOP_TDT_TOT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 168
#define RSDVSG_ATTR_TS_PLAYER_INTERFACE_SETTINGS_PACKET_LENGTH              RS_SPECIFIC_PUBLIC_ATTR_BASE + 275
#define RSDVSG_ATTR_TS_PLAYER_INTERFACE_SETTINGS_TS_SERIAL_OUTPUT           RS_SPECIFIC_PUBLIC_ATTR_BASE + 276
#define RSDVSG_ATTR_TS_PLAYER_INTERFACE_SETTINGS_STOP_DATA                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 277
#define RSDVSG_ATTR_TS_PLAYER_NULLPACKET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 278
#define RSDVSG_ATTR_TS_PLAYER_PAYLOAD                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 279
#define RSDVSG_ATTR_TS_PLAYER_PRBS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 280
#define RSDVSG_ATTR_NEW_TS_RECORDER_STREAM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 281
#define RSDVSG_ATTR_QUERY_TS_RECORDER_FREE_MEMORY                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 282
#define RSDVSG_ATTR_QUERY_TS_RECORDER_INPUT_TS_DATA_RATE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 283
#define RSDVSG_ATTR_QUERY_TS_RECORDER_RECORD_LENGTH                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 284
#define RSDVSG_ATTR_QUERY_TS_RECORDER_PROGRESS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 285
#define RSDVSG_ATTR_TS_RECORDER_INTERFACE_SETTINGS_INPUT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 286
#define RSDVSG_ATTR_TS_RECORDER_INTERFACE_SETTINGS_RECORD_SIZE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 287
#define RSDVSG_ATTR_TS_RECORDER_INTERFACE_SETTINGS_RECORD_MODE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 288
#define RSDVSG_ATTR_FILE_MANAGER_MAKE_DIRECTORY                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 289
#define RSDVSG_ATTR_FILE_MANAGER_DELETE_FILE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 290
#define RSDVSG_ATTR_FILE_MANAGER_DELETE_DIRECTORY                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 291
#define RSDVSG_ATTR_FILE_MANAGER_CHANGE_DIRECTORY                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 292
#define RSDVSG_ATTR_FILE_MANAGER_CHANGE_DRIVES                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 293
#define RSDVSG_ATTR_REFERENCE_OSCILLATOR_SOURCE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 294
#define RSDVSG_ATTR_SYSTEM_DISPLAY_UPDATE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 295
#define RSDVSG_ATTR_SYSTEM_KEYPAD_LOCK                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 296
#define RSDVSG_ATTR_SYSTEM_VERSION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 161
#define RSDVSG_ATTR_SYSTEM_ERROR                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 162
#define RSDVSG_ATTR_SYSTEM_ALL_ERRORS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 163
#define RSDVSG_ATTR_SYSTEM_PRESET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 164
#define RSDVSG_ATTR_STORE_INSTRUMENT_DATA                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 297
#define RSDVSG_ATTR_RECALL_INSTRUMENT_DATA                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 298
#define RSDVSG_ATTR_ID_QUERY_RESPONSE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 165

// rsdvsg_rngStatus
#define RSDVSG_VAL_AV_STATUS_STOP                                           0
#define RSDVSG_VAL_AV_STATUS_PAUSE                                          1
#define RSDVSG_VAL_AV_STATUS_PLAY                                           2
#define RSDVSG_VAL_AV_STATUS_RUN                                            3
#define RSDVSG_VAL_AV_STATUS_RUNN                                           4
#define RSDVSG_VAL_AV_STATUS_REC                                            5
#define RSDVSG_VAL_AV_STATUS_LOAD                                           6

// rsdvsg_rngTVStandard
#define RSDVSG_VAL_AVP_TV_STANDARD_DVB                                      0
#define RSDVSG_VAL_AVP_TV_STANDARD_ATSC                                     1

// rsdvsg_rngTimingVideoResolution
#define RSDVSG_VAL_AVP_RESOLUTION_AUTO                                      0
#define RSDVSG_VAL_AVP_RESOLUTION_1080I                                     1
#define RSDVSG_VAL_AVP_RESOLUTION_720P                                      2
#define RSDVSG_VAL_AVP_RESOLUTION_576I                                      3

// rsdvsg_rngTimingVideoAspectRatio
#define RSDVSG_VAL_AVP_ASPECT_RATIO_AUTO                                    0
#define RSDVSG_VAL_AVP_ASPECT_RATIO_16_9                                    1
#define RSDVSG_VAL_AVP_ASPECT_RATIO_4_3                                     2

// rsdvsg_rngInterfaceNTSCPedestal
#define RSDVSG_VAL_AV_NTSC_PEDESTAL_OFF                                     0
#define RSDVSG_VAL_AV_NTSC_PEDESTAL_IRE75                                   1

// rsdvsg_rngInterfaceColorComponent
#define RSDVSG_VAL_AV_COMPONENT_COLOR_RGB                                   0
#define RSDVSG_VAL_AV_COMPONENT_COLOR_YPBPR                                 1

// rsdvsg_rngInterfaceComponentHDTV
#define RSDVSG_VAL_AV_COMPONENT_HDTV_SYNC_TLEV                              0
#define RSDVSG_VAL_AV_COMPONENT_HDTV_SYNC_BLEV                              1

// rsdvsg_rngInterfaceMonitorSelect
#define RSDVSG_VAL_AV_SCART_MONITOR_INPUT_COMPOSITE                         0
#define RSDVSG_VAL_AV_SCART_MONITOR_INPUT_COMPONENT                         1
#define RSDVSG_VAL_AV_SCART_MONITOR_INPUT_Y_C                               2

// rsdvsg_rngInterfaceMonitorScartFormat
#define RSDVSG_VAL_AV_SCART_FORMAT_WSS_OFF                                  0
#define RSDVSG_VAL_AV_SCART_FORMAT_WSS_4_3                                  1
#define RSDVSG_VAL_AV_SCART_FORMAT_WSS_16_9                                 2

// rsdvsg_rngInterfaceTsAsi
#define RSDVSG_VAL_AV_TS_ASI_CONT_MODE                                      0
#define RSDVSG_VAL_AV_TS_ASI_PACK_MODE                                      1

// rsdvsg_rngInterfacePacketLength
#define RSDVSG_VAL_AV_LENGTH_P188                                           0
#define RSDVSG_VAL_AV_LENGTH_P204                                           1
#define RSDVSG_VAL_AV_LENGTH_P208                                           2

// rsdvsg_rngInterfaceRGBSync
#define RSDVSG_VAL_AV_RGB_SYNC_OFF                                          0
#define RSDVSG_VAL_AV_RGB_SYNC_GREEN                                        1

// rsdvsg_rngGeneratorVideoAdjustment
#define RSDVSG_VAL_AVG_VIDEO_ADJUSTMENT_CALIBRATED                          0
#define RSDVSG_VAL_AVG_VIDEO_ADJUSTMENT_VARIABLE                            1

// rsdvsg_rngGeneratorVideoResolClip
#define RSDVSG_VAL_AVG_VIDEO_RESOLUTION_CLIPPING_NONE                       0
#define RSDVSG_VAL_AVG_VIDEO_RESOLUTION_CLIPPING_5_BIT                      1
#define RSDVSG_VAL_AVG_VIDEO_RESOLUTION_CLIPPING_6_BIT                      2
#define RSDVSG_VAL_AVG_VIDEO_RESOLUTION_CLIPPING_7_BIT                      3
#define RSDVSG_VAL_AVG_VIDEO_RESOLUTION_CLIPPING_8_BIT                      4
#define RSDVSG_VAL_AVG_VIDEO_RESOLUTION_CLIPPING_9_BIT                      5
#define RSDVSG_VAL_AVG_VIDEO_RESOLUTION_CLIPPING_10_BIT                     6
#define RSDVSG_VAL_AVG_VIDEO_RESOLUTION_CLIPPING_11_BIT                     7

// rsdvsg_rngGeneratorAudioSource
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_MUTE                                    0
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_VIDEO                                   1
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_SINGLE_FREQ                        2
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_VARIOUS_FREQ                       3
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_CH_1                               4
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_CH_2                               5
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_CH_3                               6
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_CH_4                               7
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_CH_5                               8
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_CH_6                               9
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_CH_7                               10
#define RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_CH_8                               11

// rsdvsg_rngGeneratorAudioChAlloc
#define RSDVSG_VAL_AVG_CHAN_ALOC_00                                         0
#define RSDVSG_VAL_AVG_CHAN_ALOC_01                                         1
#define RSDVSG_VAL_AVG_CHAN_ALOC_02                                         2
#define RSDVSG_VAL_AVG_CHAN_ALOC_03                                         3
#define RSDVSG_VAL_AVG_CHAN_ALOC_04                                         4
#define RSDVSG_VAL_AVG_CHAN_ALOC_05                                         5
#define RSDVSG_VAL_AVG_CHAN_ALOC_06                                         6
#define RSDVSG_VAL_AVG_CHAN_ALOC_07                                         7
#define RSDVSG_VAL_AVG_CHAN_ALOC_08                                         8
#define RSDVSG_VAL_AVG_CHAN_ALOC_09                                         9
#define RSDVSG_VAL_AVG_CHAN_ALOC_0A                                         10
#define RSDVSG_VAL_AVG_CHAN_ALOC_0B                                         11
#define RSDVSG_VAL_AVG_CHAN_ALOC_0C                                         12
#define RSDVSG_VAL_AVG_CHAN_ALOC_0D                                         13
#define RSDVSG_VAL_AVG_CHAN_ALOC_0E                                         14
#define RSDVSG_VAL_AVG_CHAN_ALOC_0F                                         15
#define RSDVSG_VAL_AVG_CHAN_ALOC_10                                         16
#define RSDVSG_VAL_AVG_CHAN_ALOC_11                                         17
#define RSDVSG_VAL_AVG_CHAN_ALOC_12                                         18
#define RSDVSG_VAL_AVG_CHAN_ALOC_13                                         19
#define RSDVSG_VAL_AVG_CHAN_ALOC_14                                         20
#define RSDVSG_VAL_AVG_CHAN_ALOC_15                                         21
#define RSDVSG_VAL_AVG_CHAN_ALOC_16                                         22
#define RSDVSG_VAL_AVG_CHAN_ALOC_17                                         23
#define RSDVSG_VAL_AVG_CHAN_ALOC_18                                         24
#define RSDVSG_VAL_AVG_CHAN_ALOC_19                                         25
#define RSDVSG_VAL_AVG_CHAN_ALOC_1A                                         26
#define RSDVSG_VAL_AVG_CHAN_ALOC_1B                                         27
#define RSDVSG_VAL_AVG_CHAN_ALOC_1C                                         28
#define RSDVSG_VAL_AVG_CHAN_ALOC_1D                                         29
#define RSDVSG_VAL_AVG_CHAN_ALOC_1E                                         30
#define RSDVSG_VAL_AVG_CHAN_ALOC_1F                                         31

// rsdvsg_rngGeneratorAudioResolution
#define RSDVSG_VAL_AVG_AUDIO_RESOLUTION_16_BIT                              0
#define RSDVSG_VAL_AVG_AUDIO_RESOLUTION_20_BIT                              1
#define RSDVSG_VAL_AVG_AUDIO_RESOLUTION_24_BIT                              2

// rsdvsg_rngTSApplication
#define RSDVSG_VAL_TSGEN_PREPARE_PLAY                                       0
#define RSDVSG_VAL_TSGEN_PREPARE_RECORD                                     1

// rsdvsg_rngTSPCRJitterWaveform
#define RSDVSG_VAL_TSGEN_WAVEFORM_SINUS                                     0
#define RSDVSG_VAL_TSGEN_WAVEFORM_SQUARE                                    1
#define RSDVSG_VAL_TSGEN_WAVEFORM_TRIANGLE                                  2

// rsdvsg_rngTSSerialOutput
#define RSDVSG_VAL_TSGEN_ASIBURST                                           0
#define RSDVSG_VAL_TSGEN_ASICONT                                            1
#define RSDVSG_VAL_TSGEN_SMPT                                               2

// rsdvsg_rngTSInterfaceStopData
#define RSDVSG_VAL_TSGEN_STOP_DATA_NONE                                     0
#define RSDVSG_VAL_TSGEN_STOP_DATA_NULLPACKET                               1

// rsdvsg_rngTSNullpackets
#define RSDVSG_VAL_TSGEN_NULLPACKET_H184                                    0
#define RSDVSG_VAL_TSGEN_NULLPACKET_S187                                    1
#define RSDVSG_VAL_TSGEN_NULLPACKET_H200                                    2
#define RSDVSG_VAL_TSGEN_NULLPACKET_S203                                    3
#define RSDVSG_VAL_TSGEN_NULLPACKET_H204                                    4
#define RSDVSG_VAL_TSGEN_NULLPACKET_S207                                    5

// rsdvsg_rngTSPayload
#define RSDVSG_VAL_PAYLOAD_PRBS                                             0
#define RSDVSG_VAL_PAYLOAD_H00                                              1
#define RSDVSG_VAL_PAYLOAD_HFF                                              2
#define RSDVSG_VAL_PAYLOAD_COUN                                             3

// rsdvsg_rngTSPRBS
#define RSDVSG_VAL_PRBS_P231                                                0
#define RSDVSG_VAL_PRBS_P151                                                1

// rsdvsg_rngTSRecorderInterfaceInput
#define RSDVSG_VAL_TSGEN_ASIF                                               0
#define RSDVSG_VAL_TSGEN_ASIR                                               1
#define RSDVSG_VAL_TSGEN_SMPF                                               2
#define RSDVSG_VAL_TSGEN_SMPR                                               3
#define RSDVSG_VAL_TSGEN_SF8T                                               4
#define RSDVSG_VAL_TSGEN_SR8T                                               5
#define RSDVSG_VAL_TSGEN_SF8B                                               6
#define RSDVSG_VAL_TSGEN_SR8B                                               7
#define RSDVSG_VAL_TSGEN_SF10                                               8
#define RSDVSG_VAL_TSGEN_SR10                                               9

// rsdvsg_rngTSRecorderInterfaceRecMode
#define RSDVSG_VAL_TSGEN_MODE_SINGLESHOT                                    0
#define RSDVSG_VAL_TSGEN_MODE_CONTINUOUS                                    1

// rsdvsg_rngRefOscillatorSource
#define RSDVSG_VAL_ROSC_SOURCE_INT                                          0
#define RSDVSG_VAL_ROSC_SOURCE_EXT                                          1

// rsdvsg_rngInterfaceColorSystem
#define RSDVSG_VAL_COLOR_SYSTEM_NTSC                                        0
#define RSDVSG_VAL_COLOR_SYSTEM_PAL                                         1
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSDVSG_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSDVSG_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSDVSG_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSDVSG_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/

// Signal Standard	
#define RSDVSG_VAL_AV_SIGNAL_HDMI      	0
#define RSDVSG_VAL_AV_SIGNAL_HD_SDI    	1
#define RSDVSG_VAL_AV_SIGNAL_COMPOSITE 	2
#define RSDVSG_VAL_AV_SIGNAL_COMPONENT 	3
#define RSDVSG_VAL_AV_SIGNAL_YC        	4
#define RSDVSG_VAL_AV_SIGNAL_SCART     	5
#define RSDVSG_VAL_AV_SIGNAL_VGA       	6
#define RSDVSG_VAL_AV_SIGNAL_TS        	7

// File Operation
#define RSDVSG_VAL_FILE_NEW    	0
#define RSDVSG_VAL_FILE_COPY   	1
#define RSDVSG_VAL_FILE_RENAME 	2
#define RSDVSG_VAL_FILE_DELETE 	3
#define RSDVSG_VAL_FILE_RDIR   	4
#define RSDVSG_VAL_FILE_CDIR   	5
#define RSDVSG_VAL_FILE_CDISC  	6
	
// Read Catalog
#define RSDVSG_VAL_MMEM_CATALOG_FILES  	0
#define RSDVSG_VAL_MMEM_CATALOG_SUBDIR 	1
    
// Register Operation	
#define RSDVSG_VAL_PRESET_STAT_REG 	0
#define RSDVSG_VAL_QUEST_STAT_REG  	1
#define RSDVSG_VAL_OPER_STAT_REG   	2
	
// Status Registers Query
#define RSDVSG_VAL_REG_CONDITION   	0
#define RSDVSG_VAL_REG_EVENT       	1

// Application Operation	
#define RSDVSG_VAL_SAVE   	0
#define RSDVSG_VAL_RECALL 	1

/****************************************************************************
 *---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/

ViStatus _VI_FUNC rsdvsg_init (ViRsrc resourceName, ViBoolean IDQuery,
                               ViBoolean resetDevice, ViPSession instrumentHandle);
ViStatus _VI_FUNC rsdvsg_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                          ViBoolean resetDevice,
                                          ViString optionString,
                                          ViPSession instrumentHandle);
ViStatus _VI_FUNC rsdvsg_AVSignalPlayerMode (ViSession instrumentHandle);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerStatus (ViSession instrumentHandle,
                                                        ViInt32 command);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerStatus (ViSession instrumentHandle,
                                                    ViPInt32 command);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerSource (ViSession instrumentHandle,
                                                        ViString fileName);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerSourceParameters
             (ViSession instrumentHandle, ViChar _VI_FAR fileDateSize[],
              ViPReal64 originalLoopTime, ViChar _VI_FAR originalTSDataRate[]);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerFileStartStopPosition
             (ViSession instrumentHandle, ViReal64 playerFileStartPosition,
              ViReal64 playerFileStopPosition);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerCurrentPlayTime
             (ViSession instrumentHandle, ViPReal64 currentPlayTime);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerTVStandard
             (ViSession instrumentHandle, ViInt32 TVStandard);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerService (ViSession instrumentHandle,
                                                         ViInt32 service);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerServiceListMaximumNumber
             (ViSession instrumentHandle, ViPInt32 maximumNumber);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerServiceListInfo
             (ViSession instrumentHandle, ViInt32 serviceList, ViInt32 bufferSize,
              ViChar _VI_FAR info[]);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerVideo (ViSession instrumentHandle,
                                                       ViInt32 video);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerVideoListMaximumNumber
             (ViSession instrumentHandle, ViPInt32 maximumNumber);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerVideoListInfo
             (ViSession instrumentHandle, ViInt32 videoElementaryStream,
              ViInt32 bufferSize, ViChar _VI_FAR info[]);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerVideoParameters
             (ViSession instrumentHandle, ViChar _VI_FAR coding[],
              ViChar _VI_FAR ESDataRate[], ViChar _VI_FAR frameRate[],
              ViChar _VI_FAR pictureSize[], ViChar _VI_FAR pictureFormat[]);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerAudio (ViSession instrumentHandle,
                                                       ViInt32 audio);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerAudioListMaximumNumber
             (ViSession instrumentHandle, ViPInt32 maximumNumber);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerAudioListInfo
             (ViSession instrumentHandle, ViInt32 audioElementaryStream,
              ViInt32 bufferSize, ViChar _VI_FAR info[]);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerAudioParameters
             (ViSession instrumentHandle, ViChar _VI_FAR coding[],
              ViChar _VI_FAR ESDataRate[], ViPInt32 samplingRate);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerVideoResolution
             (ViSession instrumentHandle, ViInt32 videoResolution);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerVideoAspectRatio
             (ViSession instrumentHandle, ViInt32 videoAspectRatio);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceConformity
             (ViSession instrumentHandle, ViInt32 signalStandard,
              ViPBoolean conformity);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerInterfaceHDMI
             (ViSession instrumentHandle, ViBoolean HDCP, ViBoolean DVI);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceHDSDI
             (ViSession instrumentHandle, ViPInt32 resolution,
              ViPBoolean embeddedAudio);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerInterfaceComposite
             (ViSession instrumentHandle, ViInt32 NTSCPedestal,
              ViBoolean macroVision);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceComposite
             (ViSession instrumentHandle, ViPInt32 colorSystem);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerInterfaceComponent
             (ViSession instrumentHandle, ViInt32 color, ViInt32 RGBSync);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceComponent
             (ViSession instrumentHandle, ViPInt32 HDTVSync);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerInterfaceScart
             (ViSession instrumentHandle, ViInt32 monitorInputSelect,
              ViInt32 formatWSS);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceYC (ViSession instrumentHandle,
                                                         ViPInt32 colorSystem);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceVGA (ViSession instrumentHandle,
                                                          ViPInt32 colorSystem);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceTS (ViSession instrumentHandle,
                                                         ViPInt32 packetLength,
                                                         ViPInt32 ASI);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceRGBSync
             (ViSession instrumentHandle, ViPInt32 RGBSync);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerHDMIEDID (ViSession instrumentHandle,
                                                      ViInt32 bufferSize,
                                                      ViChar _VI_FAR EDIDHDMI[]);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerVGAEDID (ViSession instrumentHandle,
                                                     ViInt32 bufferSize,
                                                     ViChar _VI_FAR EDIDVGA[]);
ViStatus _VI_FUNC rsdvsg_AVSignalGeneratorMode (ViSession instrumentHandle);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorStatus
             (ViSession instrumentHandle, ViInt32 command);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorStatus (ViSession instrumentHandle,
                                                       ViPInt32 command);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorSource
             (ViSession instrumentHandle, ViString fileName);
ViStatus _VI_FUNC rsdvsg_SelectAVSignalGeneratorVideoFrame
             (ViSession instrumentHandle, ViInt32 videoFrame);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorVideoFrameRate
             (ViSession instrumentHandle, ViReal64 frameRate);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorAudioSamplingRate
             (ViSession instrumentHandle, ViInt32 samplingRate);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorPixelshift
             (ViSession instrumentHandle, ViBoolean pixelshift,
              ViInt32 verticalPixelshift);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorRGBVideo
             (ViSession instrumentHandle, ViInt32 adjustment,
              ViInt32 resolutionClipping, ViReal64 overallAmplification,
              ViReal64 rLevel, ViReal64 gLevel, ViReal64 bLevel, ViReal64 offset);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorYCrCbVideo
             (ViSession instrumentHandle, ViInt32 adjustment,
              ViInt32 resolutionClipping, ViReal64 overallAmplification,
              ViReal64 chromaAmplification, ViReal64 yLevel, ViReal64 cbPbLevel,
              ViReal64 crPrLevel, ViReal64 offset);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorAudio (ViSession instrumentHandle,
                                                          ViInt32 audioSource,
                                                          ViInt32 channelAllocation,
                                                          ViInt32 resolution,
                                                          ViReal64 audioLevel);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorAudioFrequency
             (ViSession instrumentHandle, ViInt32 channel,
              ViReal64 channelFrequency);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceConformity
             (ViSession instrumentHandle, ViInt32 signalStandard,
              ViPBoolean conformity);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorInterfaceHDMI
             (ViSession instrumentHandle, ViBoolean HDCP, ViBoolean DVI);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorInterfaceComposite
             (ViSession instrumentHandle, ViInt32 NTSCPedestal,
              ViBoolean macroVision);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceComposite
             (ViSession instrumentHandle, ViPInt32 colorSystem);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorInterfaceComponent
             (ViSession instrumentHandle, ViInt32 color, ViInt32 RGBSync);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceComponent
             (ViSession instrumentHandle, ViPInt32 HDTVSync);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceYC
             (ViSession instrumentHandle, ViPInt32 colorSystem);
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorInterfaceScart
             (ViSession instrumentHandle, ViInt32 monitorInputSelect,
              ViInt32 formatWSS);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceVGA
             (ViSession instrumentHandle, ViPInt32 colorSystem);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceTS
             (ViSession instrumentHandle, ViPInt32 packetLength, ViPInt32 ASI);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceRGBSync
             (ViSession instrumentHandle, ViPInt32 RGBSync);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorHDMIEDID (ViSession instrumentHandle,
                                                         ViInt32 bufferSize,
                                                         ViChar _VI_FAR EDIDHDMI[]);
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorVGAEDID (ViSession instrumentHandle,
                                                        ViInt32 bufferSize,
                                                        ViChar _VI_FAR EDIDVGA[]);
ViStatus _VI_FUNC rsdvsg_TSPlayerMode (ViSession instrumentHandle);
ViStatus _VI_FUNC rsdvsg_ConfigureTSApplication (ViSession instrumentHandle,
                                                 ViInt32 application);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerRecorderStatus (ViSession instrumentHandle,
                                                          ViInt32 command);
ViStatus _VI_FUNC rsdvsg_QueryTSPlayerRecorderStatus (ViSession instrumentHandle,
                                                      ViPInt32 command);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerSource (ViSession instrumentHandle,
                                                  ViString fileName);
ViStatus _VI_FUNC rsdvsg_QueryTSPlayerSourceParameters (ViSession instrumentHandle,
                                                        ViChar _VI_FAR fileDateSize[],
                                                        ViPReal64 originalLoopTime,
                                                        ViPInt32 originalTSDataRate);
ViStatus _VI_FUNC rsdvsg_QueryTSPlayerCurrentPlayTime (ViSession instrumentHandle,
                                                       ViPReal64 currentPlayTime);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerTimingSettings (ViSession instrumentHandle,
                                                          ViReal64 TSDataRate);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerPlayWindowSettings (ViSession instrumentHandle,
                                                              ViReal64 playWindowStart,
                                                              ViReal64 playWindowStop);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerByteRate (ViSession instrumentHandle,
                                                    ViReal64 byteRate);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerNullpacketStuffing
             (ViSession instrumentHandle, ViBoolean nullpacketStuffing);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerPCRJitter (ViSession instrumentHandle,
                                                     ViBoolean PCRJitter,
                                                     ViInt32 waveform,
                                                     ViReal64 amplitude,
                                                     ViReal64 frequency);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerSeamlessLoop (ViSession instrumentHandle,
                                                        ViBoolean continuityCounter,
                                                        ViBoolean PCRDTSPTS,
                                                        ViBoolean TDTTOT);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerInterfaceSettings
             (ViSession instrumentHandle, ViInt32 packetLength,
              ViInt32 TSSerialOutput);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerInterfaceSettingsStopData
             (ViSession instrumentHandle, ViInt32 stopData);
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerNullpacketSettings
             (ViSession instrumentHandle, ViInt32 nullpackets, ViInt32 payload,
              ViInt32 PRBS);
ViStatus _VI_FUNC rsdvsg_NewTSRecorderStream (ViSession instrumentHandle,
                                              ViString fileName);
ViStatus _VI_FUNC rsdvsg_QueryTSRecorderParameters (ViSession instrumentHandle,
                                                    ViPReal64 freeMemory,
                                                    ViPReal64 inputTSDataRate,
                                                    ViPReal64 recordLength);
ViStatus _VI_FUNC rsdvsg_QueryTSRecorderProgress (ViSession instrumentHandle,
                                                  ViPReal64 recordProgress);
ViStatus _VI_FUNC rsdvsg_ConfigureTSRecorderInterfaceSettings
             (ViSession instrumentHandle, ViInt32 input, ViReal64 recordSize,
              ViInt32 recordMode);
ViStatus _VI_FUNC rsdvsg_FileManagerOperations (ViSession instrumentHandle,
                                                ViInt32 operation, ViString source,
                                                ViString destination);
ViStatus _VI_FUNC rsdvsg_FileDirectoryPath (ViSession instrumentHandle,
                                            ViInt32 read, ViString directory,
                                            ViInt32 bufferSize,
                                            ViChar _VI_FAR output[],
                                            ViPInt32 returnedValues);
ViStatus _VI_FUNC rsdvsg_CatalogLength (ViSession instrumentHandle, ViInt32 read,
                                        ViPInt32 catalogLength);
ViStatus _VI_FUNC rsdvsg_ConfigureReferenceOscillator (ViSession instrumentHandle,
                                                       ViInt32 source);
ViStatus _VI_FUNC rsdvsg_SystemDisplayUpdate (ViSession instrumentHandle,
                                              ViBoolean displayUpdate);
ViStatus _VI_FUNC rsdvsg_SystemKeypadLock (ViSession instrumentHandle,
                                           ViBoolean keypadLock);
ViStatus _VI_FUNC rsdvsg_SetStatusRegister (ViSession instrumentHandle,
                                            ViInt32 registerOperation,
                                            ViInt32 enable, ViInt32 PTransition,
                                            ViInt32 NTransition);
ViStatus _VI_FUNC rsdvsg_GetStatusRegister (ViSession instrumentHandle,
                                            ViInt32 registerOperation,
                                            ViInt32 statusRegistersQuery,
                                            ViPInt32 registerValue);
ViStatus _VI_FUNC rsdvsg_SetUserDefStatusRegister (ViSession instrumentHandle,
                                                   ViInt32 registerOperation,
                                                   ViInt32 userdefinableBit,
                                                   ViInt32 enable,
                                                   ViInt32 PTransition,
                                                   ViInt32 NTransition);
ViStatus _VI_FUNC rsdvsg_GetUserDefStatusRegister (ViSession instrumentHandle,
                                                   ViInt32 registerOperation,
                                                   ViInt32 userdefinableBit,
                                                   ViInt32 statusRegistersQuery,
                                                   ViPInt32 registerValue);
ViStatus _VI_FUNC rsdvsg_SetAttributeViInt32 (ViSession instrumentHandle,
                                              ViString repeatedCapabilityName,
                                              ViUInt32 attributeID,
                                              ViInt32 attributeValue);
ViStatus _VI_FUNC rsdvsg_SetAttributeViReal64 (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViReal64 attributeValue);
ViStatus _VI_FUNC rsdvsg_SetAttributeViString (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViString attributeValue);
ViStatus _VI_FUNC rsdvsg_SetAttributeViBoolean (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViBoolean attributeValue);
ViStatus _VI_FUNC rsdvsg_SetAttributeViSession (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViSession attributeValue);
ViStatus _VI_FUNC rsdvsg_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rsdvsg_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rsdvsg_GetAttributeViInt32 (ViSession instrumentHandle,
                                              ViString repeatedCapabilityName,
                                              ViUInt32 attributeID,
                                              ViPInt32 attributeValue);
ViStatus _VI_FUNC rsdvsg_GetAttributeViReal64 (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViPReal64 attributeValue);
ViStatus _VI_FUNC rsdvsg_GetAttributeViString (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViInt32 bufferSize,
                                               ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsdvsg_GetAttributeViBoolean (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViPBoolean attributeValue);
ViStatus _VI_FUNC rsdvsg_GetAttributeViSession (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViPSession attributeValue);
ViStatus _VI_FUNC rsdvsg_CheckAttributeViInt32 (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViInt32 attributeValue);
ViStatus _VI_FUNC rsdvsg_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViReal64 attributeValue);
ViStatus _VI_FUNC rsdvsg_CheckAttributeViString (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViString attributeValue);
ViStatus _VI_FUNC rsdvsg_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                  ViString repeatedCapabilityName,
                                                  ViUInt32 attributeID,
                                                  ViBoolean attributeValue);
ViStatus _VI_FUNC rsdvsg_CheckAttributeViSession (ViSession instrumentHandle,
                                                  ViString repeatedCapabilityName,
                                                  ViUInt32 attributeID,
                                                  ViSession attributeValue);
ViStatus _VI_FUNC rsdvsg_StoreLoadApplication (ViSession instrumentHandle,
                                               ViInt32 operation, ViString source);
ViStatus _VI_FUNC rsdvsg_StoreLoadInstrumentData (ViSession instrumentHandle,
                                                  ViInt32 operation,
                                                  ViInt32 stateNumber);
ViStatus _VI_FUNC rsdvsg_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rsdvsg_self_test (ViSession instrumentHandle,
                                    ViPInt16 selfTestResult,
                                    ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsdvsg_revision_query (ViSession instrumentHandle,
                                         ViChar _VI_FAR instrumentDriverRevision[],
                                         ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsdvsg_setCheckOption (ViSession instrumentHandle,
                                         ViBoolean optionChecking);
ViStatus _VI_FUNC rsdvsg_SystemVersion (ViSession instrumentHandle, ViInt32 length,
                                        ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rsdvsg_error_query (ViSession instrumentHandle,
                                      ViPInt32 errorCode,
                                      ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsdvsg_ErrorList (ViSession instrumentHandle, ViInt32 bufferSize,
                                    ViChar _VI_FAR errors[]);
ViStatus _VI_FUNC rsdvsg_error_message (ViSession instrumentHandle,
                                        ViStatus errorCode,
                                        ViChar _VI_FAR errorMessage[]);
ViStatus  rsdvsg_SetOPCTimeout (ViSession instrumentHandle,
                                        ViInt32 timeout);
ViStatus  rsdvsg_GetOPCTimeout (ViSession instrumentHandle,
                                        ViPInt32 timeout);
ViStatus _VI_FUNC rsdvsg_GetError (ViSession instrumentHandle, ViPStatus code,
                                   ViInt32 bufferSize,
                                   ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rsdvsg_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rsdvsg_ReadToFileFromInstrument (ViSession instrumentHandle,
                                                   ViString source,
                                                   ViString destination);
ViStatus _VI_FUNC rsdvsg_WriteFromFileToInstrument (ViSession instrumentHandle,
                                                    ViString source,
                                                    ViString destination);
ViStatus _VI_FUNC rsdvsg_WriteInstrData (ViSession instrumentHandle,
                                         ViString writeBuffer);
ViStatus _VI_FUNC rsdvsg_ReadInstrData (ViSession instrumentHandle,
                                        ViInt32 numberOfBytesToRead,
                                        ViChar _VI_FAR readBuffer[],
                                        ViPUInt32 numBytesRead);
ViStatus _VI_FUNC rsdvsg_QueryViBoolean (ViSession instrumentHandle,
                                         ViString command, ViPBoolean value);
ViStatus _VI_FUNC rsdvsg_QueryViInt32 (ViSession instrumentHandle, ViString command,
                                       ViPInt32 value);
ViStatus _VI_FUNC rsdvsg_QueryViReal64 (ViSession instrumentHandle,
                                        ViString command, ViPReal64 value);
ViStatus _VI_FUNC rsdvsg_QueryViString (ViSession instrumentHandle,
                                        ViString command, ViInt32 bufferSize,
                                        ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rsdvsg_close (ViSession instrumentHandle);



/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSDVSG_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSDVSG_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSDVSG_ERROR_INCORRECT_STATE        (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSDVSG_ERROR_INCORRECT_RESPONSE     (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSDVSG_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSDVSG_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSDVSG_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSDVSG_ERRMSG_INCORRECT_STATE       "Instrument in incorrect state"
#define RSDVSG_ERRMSG_INCORRECT_RESPONSE    "Response incorrect; report problem to R&S support!"
#define RSDVSG_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSDVSG_ERROR_CODES_AND_MSGS \
    {RSDVSG_ERROR_INCORRECT_STATE,   RSDVSG_ERRMSG_INCORRECT_STATE},\
    {RSDVSG_ERROR_INCORRECT_RESPONSE,     RSDVSG_ERRMSG_INCORRECT_RESPONSE},\
    {RSDVSG_ERROR_MAX_TIME_EXCEEDED,    RSDVSG_ERRMSG_MAX_TIME_EXCEEDED}


#define VI_ERROR_INSTR_FILE_OPEN                (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE               (_VI_ERROR+0x3FFC0801L)
#define VI_ERROR_INSTR_INTERPRETING_RESPONSE    (_VI_ERROR+0x3FFC0803L)
    


/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSDVSG_HEADER */
