/*****************************************************************************
 *------------------------ Utility  -----------------------------------------*
 *****************************************************************************/
#include <float.h>
#include <math.h>
#include "rsdvsg.h"

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

ViStatus rsdvsg_CheckStatusCallback (ViSession io);
ViStatus rsdvsg_WaitForOPCCallback (ViSession io);


/*****************************************************************************
 *- Common Callback Declarations --------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsdvsg_quotedString_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Reads string 
 *           returned with apostrophes
 *****************************************************************************/
 ViStatus rsdvsg_quotedString_ReadCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, ViInt32 bufSize, void *value)
{
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
    ViStatus    error           = VI_SUCCESS;
    ViUInt32    count           = bufSize;
    ViInt32     idx             = 0;
    ViChar      *cmd            = VI_NULL,
                *buffer         = VI_NULL;

    viCheckErr (Rs_GetAttributeIndex (instrSession, attributeId, &idx));

    /* If no command available, exit the function */
    if (sessionProperties -> attr[idx].command == VI_NULL)
        return VI_SUCCESS;

    /* Check if attribute uses repeated capabilities */
    if (sessionProperties -> attr[idx].usercaps == VI_TRUE)
        {
        if (!repCapName)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);

        if (strlen (repCapName) == 0)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);
        }

    /* Build the command string */
    viCheckErr (Rs_BuildCommandString (instrSession, attributeId, idx, repCapName, &cmd));  

    /* --- If the range table for the attribute exists, revert 'command string'
       back to the 'discreteOrMinValue' and return it as attribute value. --- */

    if (sessionProperties -> attr[idx].rangeTable)
        if (sessionProperties -> attr[idx].rangeTable -> type == RS_VAL_DISCRETE)
            {
            ViUInt16    i = 0;
            ViChar      *p2string;
            ViInt32     found;

            viCheckAlloc (buffer = (ViChar*) malloc (RS_MAX_MESSAGE_BUF_SIZE));
            count = RS_MAX_MESSAGE_BUF_SIZE;

            if (sessionProperties -> attr[idx].access == RS_VAL_READ_WRITE)
            {
                viCheckErr (viPrintf(instrSession, "%s?\n", cmd));
                viCheckErr (viScanf(instrSession, "\"%#[^\"]\"", &count, buffer));
            }
            else
            {
                viCheckErr (viPrintf(instrSession, "%s\n", cmd));
                viCheckErr (viScanf(instrSession, "\"%#[^\"]\"", &count, buffer));
            }

            /* Go over range table values and check if the command string is present */
            do
                {
                p2string = sessionProperties -> attr[idx].rangeTable -> rangeValues[i].cmdString;

                if (sessionProperties -> attr[idx].rangeTable -> rangeValues[i].cmdString == RS_RANGE_TABLE_END_STRING)
                    return RS_ERROR_INVALID_VALUE;
#ifdef RS_LONG_RESPONSE            
                if ((strlen (p2string) <= 3) || (strlen (buffer) > 4))
                    found = strcmp (buffer,p2string);
                else
                    found = strncmp (buffer, p2string, strlen (p2string));
#else
                found = strcmp (buffer,p2string);
#endif
                if (found == 0)
                    break;

                i++;

                } while (i);

            /* Return value from range table */
            switch (sessionProperties -> attr[idx].dataType)
                {
                case RS_VAL_INT32:
                    *(ViInt32 *)value = sessionProperties -> attr[idx].rangeTable -> rangeValues[i].discreteOrMinValue;
                    break;
                case RS_VAL_REAL64:
                    *(ViReal64 *)value = sessionProperties -> attr[idx].rangeTable -> rangeValues[i].discreteOrMinValue;
                    break;
                case RS_VAL_STRING:
                case RS_VAL_BOOLEAN:
                case RS_VAL_ADDR:
                case RS_VAL_SESSION:
                case RS_VAL_EVENT:
                case RS_VAL_UNKNOWN_TYPE:
                default:
                    viCheckErr (RS_ERROR_INVALID_TYPE);
                }
            
            goto Error;
            }

        /* --- Attribute data type is used for VISA I/O --- */
        switch (sessionProperties -> attr[idx].dataType)
            {
            case RS_VAL_STRING:
                if (sessionProperties -> attr[idx].access == RS_VAL_READ_WRITE)
                {
                    viCheckErr (viPrintf(instrSession, "%s?\n", cmd));
                    viCheckErr (viScanf(instrSession, "\"%#[^\"]\"", &count, value));
                }
                else
                {
                    viCheckErr (viPrintf(instrSession, "%s\n", cmd));
                    viCheckErr (viScanf(instrSession, "\"%#[^\"]\"", &count, value));
                }
                break;
            case RS_VAL_INT32:
            case RS_VAL_REAL64:                
            case RS_VAL_BOOLEAN:
            case RS_VAL_ADDR:
            case RS_VAL_SESSION:
            case RS_VAL_EVENT:
            case RS_VAL_UNKNOWN_TYPE:
            default:
                viCheckErr (RS_ERROR_INVALID_TYPE);
            }

Error:

    if (buffer)
        free (buffer);

    if (cmd)
        free (cmd);

    return error;
}

/*****************************************************************************
 * Function: rsdvsg_quotedString_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Writes string 
 *           with apostrophes
 *****************************************************************************/
ViStatus rsdvsg_quotedString_WriteCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, void *value)
{
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
    ViStatus    error               = VI_SUCCESS;
    ViInt32     idx                 = 0;
    ViChar      *cmd                = VI_NULL;

    viCheckErr (Rs_GetAttributeIndex (instrSession, attributeId, &idx));

    /* If no command available, exit the function */
    if (sessionProperties -> attr[idx].command == VI_NULL)
        return VI_SUCCESS;

    /* Check if attribute uses repeated capabilities */
    if (sessionProperties -> attr[idx].usercaps == VI_TRUE)
        {
        if (!repCapName)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);

        if (strlen (repCapName) == 0)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);
        }

    /* Build the command string */
    viCheckErr (Rs_BuildCommandString (instrSession, attributeId, idx, repCapName, &cmd));  

    /* --- If the range table for the attribute exists, pass 'command string' or
       'discreteOrMinValue' instead of attribute value. --- */

    if (sessionProperties -> attr[idx].rangeTable)
        if (sessionProperties -> attr[idx].rangeTable -> type == RS_VAL_DISCRETE)
            {
            ViUInt16    i = 0;
            ViChar      *p2string;
            ViReal64    *p2discrete;
            ViInt32     data_type = sessionProperties -> attr[idx].dataType;
            ViReal64    precision = sessionProperties -> attr[idx].comparePrecision;

            precision = (precision < DBL_MIN) ? DBL_MIN : precision;

            /* Allowed data types are only ViInt32 or ViReal64 */
            switch (data_type)
                {
                case RS_VAL_INT32:
                case RS_VAL_REAL64:
                    /* Valid data types */
                    break;
                case RS_VAL_STRING:
                case RS_VAL_BOOLEAN:
                case RS_VAL_EVENT:
                case RS_VAL_ADDR:
                case RS_VAL_SESSION:
                case RS_VAL_UNKNOWN_TYPE:
                default:
                    viCheckErr (RS_ERROR_INVALID_TYPE);
                }

            /* Go over range table values and find appropriate command string for passed
               attribute value. For floating point values use predefined attribute precision. */
            do
                {
                p2discrete = &sessionProperties -> attr[idx].rangeTable -> rangeValues[i].discreteOrMinValue;
                p2string = sessionProperties -> attr[idx].rangeTable -> rangeValues[i].cmdString;

                if (sessionProperties -> attr[idx].rangeTable -> rangeValues[i].cmdString == RS_RANGE_TABLE_END_STRING)
                    return RS_ERROR_INVALID_VALUE;

                if (data_type == RS_VAL_INT32)
                    if (fabs(*p2discrete - *(ViInt32 *)value) <= DBL_MIN)
                        {
                        viCheckErr (viPrintf(instrSession, "%s \'%s\'\n", cmd, p2string));
                        break;
                        }

                if (data_type == RS_VAL_REAL64)
                    if (fabs(*p2discrete - *(ViReal64 *)value) <= precision)
                        {
                        viCheckErr (viPrintf(instrSession, "%s \'%s\'\n", cmd, p2string));
                        break;
                        }

                i++;

                } while (i);

            goto Error; /* Exit the function */
            }

    /* --- Attribute data type is used for VISA I/O --- */
    switch (sessionProperties -> attr[idx].dataType)
        {
        case RS_VAL_INT32:
            viCheckErr (viPrintf(instrSession, "%s %ld\n", cmd, *(ViInt32 *)value));
            break;
        case RS_VAL_REAL64:
            viCheckErr (viPrintf(instrSession, "%s %.12f\n", cmd, *(ViReal64 *)value));
            break;
        case RS_VAL_STRING:
            viCheckErr (viPrintf(instrSession, "%s \'%s\'\n", cmd, value));
            break;
        case RS_VAL_BOOLEAN:
            viCheckErr (viPrintf(instrSession, "%s %s\n", cmd, (*(ViBoolean *)value == VI_FALSE) ? "OFF" : "ON"));
            break;
        case RS_VAL_EVENT:
            viCheckErr (viPrintf(instrSession, "%s\n", cmd));
            break;
        case RS_VAL_ADDR:
        case RS_VAL_SESSION:
        case RS_VAL_UNKNOWN_TYPE:
        default:
            viCheckErr (RS_ERROR_INVALID_TYPE);
        }

Error:

    if (cmd)
        free (cmd);

    return error;

}

/*****************************************************************************
 * Function: rsdvsg_InitAttributes
 * Purpose:  This function inits attributes to the desired values if needed.
 *****************************************************************************/
ViStatus rsdvsg_InitAttributes (ViSession instrSession)
{
    ViStatus    error           = VI_SUCCESS;
    ViInt32     p2value         = 0;
    void        *p2void;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE],
                revision[RS_MAX_MESSAGE_BUF_SIZE],
                manufacturer[RS_MAX_MESSAGE_BUF_SIZE],
                model[RS_MAX_MESSAGE_BUF_SIZE],
                tmp_model[RS_MAX_MESSAGE_BUF_SIZE],
                idQ[RS_MAX_MESSAGE_BUF_SIZE],
                options_list[RS_MAX_MESSAGE_BUF_SIZE];

    buffer[RS_MAX_MESSAGE_BUF_SIZE -1] = 0;

    /*- Misc -*/

    if (!Rs_Simulating(instrSession))
        {
        /* Instrument identification */
        viCheckErr (viQueryf (instrSession, "*IDN?\n", "%[^\r\n]", idQ));
        idQ[RS_MAX_MESSAGE_BUF_SIZE - 1] = 0;
        checkErr (Rs_SetAttribute (instrSession, "", RSDVSG_ATTR_ID_QUERY_RESPONSE, 0, idQ));
        
        /* Option(s) string */
        viCheckErr (viQueryf (instrSession, "*OPT?\n", "%[^\r\n]", options_list));
        options_list[RS_MAX_MESSAGE_BUF_SIZE - 1] = 0;
        }
    else /* Simulating */
        {
        /* Instrument identification */
        strncpy (idQ, RSDVSG_SIMULATION_ID_QUERY, RS_MAX_MESSAGE_BUF_SIZE - 1);
        checkErr (Rs_SetAttribute (instrSession, "", RSDVSG_ATTR_ID_QUERY_RESPONSE, 0, idQ));
        
        /* Option(s) string */
        strncpy (options_list, RSDVSG_SIMULATION_OPT_QUERY, RS_MAX_MESSAGE_BUF_SIZE - 1);
        }

    /*- Inherent Instrument Specific Attributes ----------------------------*/

    /*- User Options -*/

    /*
    RS_ATTR_RANGE_CHECK             ... Rs_SpecificDriverNew
    RS_ATTR_QUERY_INSTRUMENT_STATUS ... Rs_SpecificDriverNew
    RS_ATTR_CACHE                   ... Rs_SpecificDriverNew
    RS_ATTR_SIMULATE                ... Rs_SpecificDriverNew
    RS_ATTR_RECORD_COERCIONS        ... not supported
    RS_ATTR_INTERCHANGE_CHECK       ... not supported
    RS_ATTR_SPY                     ... not supported
    RS_ATTR_USE_SPECIFIC_SIMULATION ... not supported
    */

    /*- Class Driver Identification -*/

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_DESCRIPTION, 0, RSDVSG_CLASS_DRIVER_DESCRIPTION));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_PREFIX, 0, RSDVSG_CLASS_DRIVER_PREFIX));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_VENDOR, 0, RSDVSG_CLASS_DRIVER_VENDOR));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_REVISION, 0, RSDVSG_CLASS_DRIVER_REVISION));
    p2value = RSDVSG_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, &p2value));
    p2value = RSDVSG_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, &p2value));

    /*- Driver Identification -*/
                                                        
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSDVSG_SPECIFIC_DRIVER_DESCRIPTION));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSDVSG_SPECIFIC_DRIVER_PREFIX));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSDVSG_SPECIFIC_DRIVER_LOCATOR));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSDVSG_SPECIFIC_DRIVER_VENDOR));

    sprintf (buffer, 
             "Driver: %s %.ld.%ld.%ld, Compiler: %s %3.2f, "
             "Components: RSEngine %.2f, VISA-Spec %.2f",
             RSDVSG_SPECIFIC_DRIVER_PREFIX,
             RSDVSG_MAJOR_VERSION,RSDVSG_MINOR_VERSION,RSDVSG_MINOR_MINOR_VERSION,
             RS_COMPILER_NAME, RS_COMPILER_VER_NUM, 
             RS_ENGINE_MAJOR_VERSION + RS_ENGINE_MINOR_VERSION/10.0, 
             Rs_ConvertVISAVer(VI_SPEC_VERSION));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, buffer));
    p2value = RSDVSG_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, &p2value));
    p2value = RSDVSG_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, &p2value));

    /*- Driver Capabilities -*/
                                   
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSDVSG_SUPPORTED_INSTRUMENT_MODELS));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSDVSG_GROUP_CAPABILITIES));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSDVSG_FUNCTION_CAPABILITIES));
    /* RS_ATTR_CHANNEL_COUNT        ... Rs_BuildRepCapTable */

    /*- Driver Setup -*/

    /* RS_ATTR_DRIVER_SETUP         ... Rs_SpecificDriverNew */

    /*- Instrument Identification -*/

    checkErr (Rs_GetAttribute (instrSession, "", RSDVSG_ATTR_ID_QUERY_RESPONSE, 0, RSDVSG_IO_BUFFER_SIZE, idQ));
    sscanf (idQ, "%[^,],%[^,],%*[^,],%256[^\n]", manufacturer, tmp_model, revision);
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_INSTRUMENT_MANUFACTURER, 0, manufacturer));
    sscanf (tmp_model,"%[^-]",model);
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_INSTRUMENT_MODEL, 0, model));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, revision));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPTIONS_LIST, 0, options_list));

    /*- Advanced Session Information -*/
                                  
    /*  
    RS_ATTR_IO_RESOURCE_DESCRIPTOR  ... rsdvsg_InitWithOptions
    RS_ATTR_LOGICAL_NAME            ... not supported
    */

    /*- Inherent Vendor Specific Attributes --------------------------------*/

    /*- Error Info -*/

    /*
    RS_ATTR_PRIMARY_ERROR           ... Rs_SetErrorInfo
    RS_ATTR_SECONDARY_ERROR         ... Rs_SetErrorInfo
    RS_ATTR_ERROR_ELABORATION       ... Rs_SetErrorInfo
    */

    /*- Session I/O -*/

    /*
    RS_ATTR_VISA_RM_SESSION         ... rsdvsg_RsInit
    RS_ATTR_IO_SESSION              ... Rs_UpdateIOSession
    RS_ATTR_IO_SESSION_TYPE         ... not supported
    */

    /*- Version Info -*/                     

    p2value = RSDVSG_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, &p2value));
    p2value = RSDVSG_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, &p2value));
    p2value = RSDVSG_MINOR_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, &p2value));
    p2value = RSDVSG_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, &p2value));
    p2value = RSDVSG_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, &p2value));
    p2value = RS_ENGINE_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_MAJOR_VERSION, 0, &p2value));
    p2value = RS_ENGINE_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_MINOR_VERSION, 0, &p2value));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_REVISION, 0, RS_ENGINE_REVISION));

    /*- Session Callbacks -*/                

    p2void = rsdvsg_WaitForOPCCallback;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPC_CALLBACK, 0, &p2void));
    //p2value = (ViInt32)p2void;
    //checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPC_CALLBACK, 0, &p2value));
    p2void = rsdvsg_CheckStatusCallback;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CHECK_STATUS_CALLBACK, 0, &p2void));
	//p2value = (ViInt32)p2void;
    //checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CHECK_STATUS_CALLBACK, 0, &p2value));
    p2value = RSDVSG_OPC_TIMEOUT;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPC_TIMEOUT, 0, &p2value));

    p2value = VI_TRUE;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, &p2value));

    /*----------------------------------------------------------------------*/
Error:
    return error;
}

/*****************************************************************************
 * Function: rsdvsg_DefaultInstrSetup
 * Purpose:  This function sends a default setup to the instrument. The
 *           rsdvsg_reset function calls this function. The
 *           rsdvsg_RsInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter. This function is
 *           useful for configuring settings that other instrument driver
 *           functions require.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsdvsg_DefaultInstrSetup (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    /* Set all the attributes to the default state. Do not update inherent attributes! */
    checkErr (Rs_ApplyAttributeDefaults (instrSession, VI_FALSE));

    /* Init attributes */
    checkErr( rsdvsg_InitAttributes (instrSession));

    if (!Rs_Simulating(instrSession))
    {
        viCheckErr( viPrintf (instrSession, "*CLS;*ESE %hu;*SRE 32", IEEE_488_2_ESE_MASK));
    }

Error:
    return error;
}

/*****************************************************************************
 * Function: rsdvsg_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           rsdvsg_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsdvsg_CheckStatus (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (Rs_QueryInstrStatus (instrSession) && !Rs_Simulating (instrSession))
        {
        checkErr( rsdvsg_CheckStatusCallback (instrSession));
        }

Error:
    return error;
}

/*****************************************************************************
 * Function: rsdvsg_WaitForOPC
 * Purpose:  This function waits for the instrument to complete the
 *           execution of all pending operations. This function is a
 *           wrapper for the WaitForOPCCallback. It can be called from
 *           other instrument driver functions.
 *
 *           The maxTime parameter specifies the maximum time to wait for
 *           operation complete in milliseconds.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsdvsg_WaitForOPC (ViSession instrSession, ViUInt32 maxTime)
{
    ViStatus    error = VI_SUCCESS;
    
    if (!Rs_Simulating(instrSession))
        {
        ViInt32  oldOPCTimeout;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSDVSG_ATTR_OPC_TIMEOUT, 0,
                                   sizeof (oldOPCTimeout), &oldOPCTimeout));

        Rs_SetAttribute (instrSession, VI_NULL, RSDVSG_ATTR_OPC_TIMEOUT, 0, &maxTime);

        error = rsdvsg_WaitForOPCCallback (instrSession);

        Rs_SetAttribute (instrSession, VI_NULL, RSDVSG_ATTR_OPC_TIMEOUT, 0, &oldOPCTimeout);

        viCheckErr( error);
        }
Error:
    return error;
}
 
/*===========================================================================*/
/* Function: String Index                                                    */
/* Purpose:  Returns the index of string in string array                     */
/*===========================================================================*/
ViInt32 rsdvsg_StringIndex(ViString array[], ViChar buff[])
{
    ViInt32 index = 0;

    while (array[index]){
        if (!strcmp(buff, array[index]))
            break; 
        index++;    
    }        
    /* If found string is not found returns -1. */
    if (!array[index]) return -1;

    return index;
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsdvsg_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsdvsg_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rsdvsg_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rsdvsg_invalidViBooleanRange (ViBoolean val)
{
    return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Set OPC Timeout                                                 */
/* Purpose:  Sets new OPC timeout                                            */
/*===========================================================================*/
ViStatus rsdvsg_SetOPCTimeout(ViSession   instrSession, ViInt32 timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0, &timeout));  
    
Error:    
    return error;
}

/*===========================================================================*/
/* Function: Get OPC Timeout                                                 */
/* Purpose:  Gets new OPC timeout                                            */
/*===========================================================================*/
ViStatus rsdvsg_GetOPCTimeout(ViSession   instrSession, ViInt32 *timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0, sizeof(timeout), timeout));  
    
Error:    
    return error;
}

/*****************************************************************************
 * Function: rsdvsg_CheckBeforeRead
 * Purpose:  This function waits for the instrument to complete the
 *           execution of all pending operations and returns data. 
 *
 *           The maxTime parameter specifies the maximum time to wait for
 *           operation complete in milliseconds.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsdvsg_CheckBeforeRead (ViSession instrSession, ViUInt32 maxTime)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt16    response    = 0;
    ViUInt32    counter     = 0;
    ViUInt16    interface   = VI_INTF_GPIB;
 
    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));

    /* Interface specific operations */
    switch (interface)
        {
        case VI_INTF_ASRL: /* Serial interface */

            /* Not supported */

        break;
        default: /* Default case */

            /* Check STB for errors. */
            do
            {
                error = viReadSTB (instrSession, &response);

                Rs_Delay (0.001);
                counter++;

                if (counter >= maxTime)
                    return VI_ERROR_TMO;

                /* Wait while STB is non zero value. Only Error/Event Queue
                   and ESB bits are taken into consideration. */
            
            } while ((response & 0x14) == 0);

            /* (check Error(Event) Queue bit in STB) */
            /* sometimes instrument sets both bits, but error queue is empty */
            return (((response & 0x4) != 0) && ((response & 0x10) == 0)) ? RS_ERROR_INSTRUMENT_STATUS : error;
        }

Error:
    return error;
}

/*****************************************************************************
 * Function: rsdvsg_GetCatalog
 * Purpose:  This function queries catalog data
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_GetCatalog(
    ViSession   instrSession,
    ViString    command,
    ViInt32     timeout,
    ViInt32     arraySize,
    ViChar      catalog[]
    )
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer [150] = "";
    ViChar*  ext_buf=VI_NULL;
    ViUInt32 ret_cnt = 0;

    checkErr (viPrintf (instrSession, "*CLS\n"));

    strcpy (buffer, command);
    strcat (buffer, ";*OPC\n");
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    
    checkErr (rsdvsg_CheckBeforeRead (instrSession, timeout));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));
    strncpy (catalog, ext_buf, arraySize);

    checkErr( rsdvsg_CheckStatus (instrSession));
Error:
    free (ext_buf);
    return error;
}

/*===========================================================================*/
/* Function: Swap 32bit Integer                                              */
/* Purpose:  Byte order for binary transfers depends on MOTOROLA or INTEL    */
/*           formats.                                                        */
/*===========================================================================*/
void rsdvsg_SWAP_32(unsigned char *data, long points)
{
    /* MOTOROLA (byte order 0123) or INTEL (byte order 3210). */
    unsigned int    word = 0x1122;
    unsigned char   *p2word = (unsigned char*)&word;

    if (*p2word == 0x11) /* MOTOROLA format */
    {
       unsigned char    src[4];
       long             *p2data = (long*)data;
       int              idx;

       for (idx = 0; idx < points; idx++, p2data++)
          {
              *((long *)src) = *(p2data);
              ((unsigned char *)(p2data))[0] =  ((unsigned char*)(src))[3];
              ((unsigned char *)(p2data))[1] =  ((unsigned char*)(src))[2];
              ((unsigned char *)(p2data))[2] =  ((unsigned char*)(src))[1];
              ((unsigned char *)(p2data))[3] =  ((unsigned char*)(src))[0];
          }
    }

   return;
}

/*****************************************************************************
 * Function: rsdvsg_atof
 * Purpose:  This function converts a string to float number. Instrument
 *           may return also string indicating invalid value
 *****************************************************************************/
ViReal64 rsdvsg_atof (ViChar *p2buf)
{
    ViReal64 value = 0.0;
    ViInt32  returned = 0;
    
    returned = sscanf (p2buf, "%le", &value);
    
    if (returned == 0)
        value = RSDVSG_NAN;
    
    return value;
}

/*****************************************************************************
 * Function: rsdvsg_dashString_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Reads dashes 
 *           returned
 *****************************************************************************/
 ViStatus rsdvsg_dashString_ReadCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, ViInt32 bufSize, void *value)
{
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
    ViStatus    error           = VI_SUCCESS;
    ViUInt32    count           = bufSize;
    ViInt32     idx             = 0;
    ViChar      *cmd            = VI_NULL,
                *buffer         = VI_NULL;
	ViUInt32    local_count		= 0;
	
    viCheckErr (Rs_GetAttributeIndex (instrSession, attributeId, &idx));

    /* If no command available, exit the function */
    if (sessionProperties -> attr[idx].command == VI_NULL)
        return VI_SUCCESS;

    /* Check if attribute uses repeated capabilities */
    if (sessionProperties -> attr[idx].usercaps == VI_TRUE)
        {
        if (!repCapName)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);

        if (strlen (repCapName) == 0)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);
        }

    /* Build the command string */
    viCheckErr (Rs_BuildCommandString (instrSession, attributeId, idx, repCapName, &cmd));  

    /* --- If the range table for the attribute exists, revert 'command string'
       back to the 'discreteOrMinValue' and return it as attribute value. --- */

    if (sessionProperties -> attr[idx].rangeTable)
        if (sessionProperties -> attr[idx].rangeTable -> type == RS_VAL_DISCRETE)
        {
            ViUInt16    i = 0;
            ViChar      *p2string;
            ViInt32     found;
			
            viCheckAlloc (buffer = (ViChar*) malloc (RS_MAX_MESSAGE_BUF_SIZE));
            count = RS_MAX_MESSAGE_BUF_SIZE;

            if (sessionProperties -> attr[idx].access == RS_VAL_READ_WRITE)
			{
				viCheckErr (viPrintf(instrSession, "%s?\n", cmd));
			}
            else
			{
				viCheckErr (viPrintf(instrSession, "%s\n", cmd));
			}
            	
			viCheckErr (Rs_ReadDataUnknownLength(instrSession, &buffer, &local_count));
			
			if ((strncmp (buffer, "---", 3)) == 0 || (strncmp (buffer, "n/a", 3)) == 0)
			{
				switch (sessionProperties -> attr[idx].dataType)
                {
	                case RS_VAL_INT32:
	                    *(ViInt32 *)value = RSDVSG_NAN_VI_INT32;
	                    break;
	                case RS_VAL_REAL64:
	                    *(ViReal64 *)value = RSDVSG_NAN;
	                    break;
	                case RS_VAL_BOOLEAN:
						*(ViBoolean *)value = VI_FALSE;
	                    break;
	                case RS_VAL_STRING:
	                case RS_VAL_ADDR:
	                case RS_VAL_SESSION:
	                case RS_VAL_EVENT:
	                case RS_VAL_UNKNOWN_TYPE:
	                default:
	                    viCheckErr (RS_ERROR_INVALID_TYPE);
                }

            	goto Error;
            }
			    
            /* Go over range table values and check if the command string is present */
            do
                {
                p2string = sessionProperties -> attr[idx].rangeTable -> rangeValues[i].cmdString;

                if (sessionProperties -> attr[idx].rangeTable -> rangeValues[i].cmdString == RS_RANGE_TABLE_END_STRING)
                    return RS_ERROR_INVALID_VALUE;

				found = strncmp (buffer, p2string, local_count - 1);

                if (found == 0)
                    break;

                i++;

                } while (i);

            /* Return value from range table */
            switch (sessionProperties -> attr[idx].dataType)
                {
                case RS_VAL_INT32:
                    *(ViInt32 *)value = sessionProperties -> attr[idx].rangeTable -> rangeValues[i].discreteOrMinValue;
                    break;
                case RS_VAL_REAL64:
                    *(ViReal64 *)value = sessionProperties -> attr[idx].rangeTable -> rangeValues[i].discreteOrMinValue;
                    break;
                case RS_VAL_STRING:
                case RS_VAL_BOOLEAN:
                case RS_VAL_ADDR:
                case RS_VAL_SESSION:
                case RS_VAL_EVENT:
                case RS_VAL_UNKNOWN_TYPE:
                default:
                    viCheckErr (RS_ERROR_INVALID_TYPE);
                }
            
            goto Error;
		}

        /* --- Attribute data type is used for VISA I/O --- */
        switch (sessionProperties -> attr[idx].dataType)
            {
            case RS_VAL_INT32:
            case RS_VAL_REAL64:
			case RS_VAL_BOOLEAN:
            	if (sessionProperties -> attr[idx].access == RS_VAL_READ_WRITE)
				{
	                viCheckErr (viPrintf(instrSession, "%s?\n", cmd));
	            }
				 else
				{
					viCheckErr (viPrintf(instrSession, "%s\n", cmd));
	            }
				   
				viCheckErr (Rs_ReadDataUnknownLength(instrSession, &buffer, &local_count));
				
				if ((strncmp (buffer, "---", 3)) == 0 || (strncmp (buffer, "n/a", 3)) == 0)
				{
					switch (sessionProperties -> attr[idx].dataType)
	                {
		                case RS_VAL_INT32:
		                    *(ViInt32 *)value = RSDVSG_NAN_VI_INT32;
		                    break;
		                case RS_VAL_REAL64:
		                    *(ViReal64 *)value = RSDVSG_NAN;
		                    break;
		                case RS_VAL_BOOLEAN:
							*(ViBoolean *)value = VI_FALSE;
		                    break;
		                case RS_VAL_STRING:
		                case RS_VAL_ADDR:
		                case RS_VAL_SESSION:
		                case RS_VAL_EVENT:
		                case RS_VAL_UNKNOWN_TYPE:
		                default:
		                    viCheckErr (RS_ERROR_INVALID_TYPE);
	                }
    
	            	goto Error;
	            }
				switch (sessionProperties -> attr[idx].dataType)
	            {
	                case RS_VAL_INT32:
	                    *(ViInt32 *)value =  atoi (buffer);
	                    break;
	                case RS_VAL_REAL64:
	                    *(ViReal64 *)value = atof (buffer);
	                    break;
	                case RS_VAL_BOOLEAN:
						*(ViBoolean *)value =  atoi (buffer) != 0;
	                    break;
	                case RS_VAL_STRING:
	                case RS_VAL_ADDR:
	                case RS_VAL_SESSION:
	                case RS_VAL_EVENT:
	                case RS_VAL_UNKNOWN_TYPE:
	                default:
	                    viCheckErr (RS_ERROR_INVALID_TYPE);
	            }
			    break;
            case RS_VAL_STRING:
			case RS_VAL_ADDR:
            case RS_VAL_SESSION:
            case RS_VAL_EVENT:
            case RS_VAL_UNKNOWN_TYPE:
            default:
                viCheckErr (RS_ERROR_INVALID_TYPE);
            }

Error:

    if (buffer)
        free (buffer);

    if (cmd)
        free (cmd);

    return error;
}

/*****************************************************************************
 * Function: rsdvsg_dashInt_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns NAN if
 *           instrument returns "---" or "n/a".
 *****************************************************************************/
ViStatus rsdvsg_dashInt_ReadCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, ViInt32 bufSize, void *value)
{
	return rsdvsg_dashString_ReadCallback(instrSession, repCapName, attributeId, bufSize, value);
}

/*****************************************************************************
 * Function: rsdvsg_dashReal_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns NAN if
 *           instrument returns "---" or "n/a".
 *****************************************************************************/
ViStatus rsdvsg_dashReal_ReadCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, ViInt32 bufSize, void *value)
{
	return rsdvsg_dashString_ReadCallback(instrSession, repCapName, attributeId, bufSize, value);
}

/*****************************************************************************
 * Function: rsdvsg_dashBoolean_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns VI_FALSE
 *           if instrument returns "---" or "n/a".
 *****************************************************************************/
ViStatus rsdvsg_dashBoolean_ReadCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, ViInt32 bufSize, void *value)
{
	return rsdvsg_dashString_ReadCallback(instrSession, repCapName, attributeId, bufSize, value);
}

